@@ -77,14 +77,15 @@ class Simulation(platrock.Common.Simulations.Simulation):
 	"""
 	webui_typename="PlatRock 3D"
 	valid_input_geojson_attrs=np.array([
-		["number",	"Rocks count",						int,	1,		1000	],
-		["z",		"Drop height (m)",					float,	2,		50		],
-		["volume",	"Rocks volume (m<sup>3</sup>)",		float,	0.1,	10		],
-		["density",	"Rocks density (kg/m<sup>3</sup>)",	float,	100,	10000	],
-		["vz",		"Vertical velocity (m/s)",			float,	-10,	0		],
-		["length",	"Length (m)",						float,	0.1,	3		],
-		["width",	"Width (m)",						float,	0.1,	3		],
-		["height",	"Height (m)",						float,	0.1,	3		]
+		["number",			"Rocks count",							int,	1,		1000	],
+		["z",				"Drop height (m)",						float,	2,		50		],
+		["rocks_min_vol",	"Min rocks volume (m<sup>3</sup>)",		float,	0.1,	10		],
+		["rocks_max_vol",	"Max rocks volume (m<sup>3</sup>)",		float,	0.1,	10		],
+		["density",			"Rocks density (kg/m<sup>3</sup>)",		float,	100,	10000	],
+		["vz",				"Vertical velocity (m/s)",				float,	-10,	0		],
+		["length",			"Length (m)",							float,	0.1,	3		],
+		["width",			"Width (m)",							float,	0.1,	3		],
+		["height",			"Height (m)",							float,	0.1,	3		]
 	])
 	def __init__(self, dt=0.02, engines=None,rocks_start_params_geojson=None,checkpoints_geojson=None,**kwargs):
 		super(Simulation, self).__init__(**kwargs)
@@ -108,11 +109,19 @@ class Simulation(platrock.Common.Simulations.Simulation):
 		self.automatic_generate_forest={"enable":False, "density":0, "dhp_mean":30, "dhp_std":10}
 		self.limit_fps=30 #used only with 3D view
 		self._geojson_polygon_offset_applied=False
-
+		self.pp=None #this will be set to Postprocessings.Postprocessing(self) later
+	
+	def __init_arrays__(self):
+		for start_zone in self.rocks_start_params:
+			params=start_zone["params"]
+			rocks_volumes = self.random_generator.rand(params["number"]) * ( params["rocks_max_vol"] - params["rocks_min_vol"] ) + params["rocks_min_vol"]
+			params["rocks_volumes"]=rocks_volumes
+	
 	def set_rocks_start_params_from_geojson(self,rocks_start_params_geojson):
 		try:
 			self._geojson_polygon_offset_applied=False
 			self.rocks_start_params=[]
+			self.nb_rocks=0
 			shp_file=gdal.OpenEx(rocks_start_params_geojson)
 			for feat in shp_file.GetLayer(0):
 				feat_dict=feat.ExportToJson(as_object=True) #loop on rocks start polygons
@@ -128,6 +137,9 @@ class Simulation(platrock.Common.Simulations.Simulation):
 					if(k not in Simulation.valid_input_geojson_attrs[:,0]):
 						rocks_properties.pop(k)
 				self.rocks_start_params[-1]["params"]=rocks_properties
+				#set the simulation total number of rocks
+				self.nb_rocks+=self.rocks_start_params[-1]["params"]["number"]
+			self.__init_arrays__()
 			return 0
 		except Exception as e:
 			message="The importation of rocks start params from geojson failed:"+str(e)
@@ -175,7 +187,7 @@ class Simulation(platrock.Common.Simulations.Simulation):
 		if not self.terrain.soil_params:
 			return False
 		for params in [rsp["params"] for rsp in self.rocks_start_params]:
-			for p in ["volume","z","vz","number"]:
+			for p in ["rocks_min_vol","rocks_max_vol","z","vz","number"]:
 				if not (p in params):
 					return False
 		for params in [sp["params"] for sp in self.terrain.soil_params]:
@@ -204,13 +216,14 @@ class Simulation(platrock.Common.Simulations.Simulation):
 		self.forest_impact_model=None
 		current_rock=self.current_rock
 		self.current_rock=None
+		pp=self.pp
+		self.pp=None
 		if(self.terrain):
 			terrain=copy.copy(self.terrain)
 			self.terrain.faces=[]
 			self.terrain.points=[]
 			self.terrain.trees=[]
 			self.terrain.faces_xyz_bb=None
-			self.terrain.Z_raster.data={"Z":self.terrain.Z_raster.data["Z"]}	#only keep the Z data
 		checkpoints=self.checkpoints[:]
 		self.checkpoints=[Objects.Checkpoint(c.path) for c in checkpoints] #NOTE: keep the checkpoints position, drop the data
 		super(Simulation,self).save_to_file()	#actual file write is here
@@ -221,6 +234,7 @@ class Simulation(platrock.Common.Simulations.Simulation):
 			self.terrain=terrain
 		self.checkpoints=checkpoints
 		self.current_rock=current_rock
+		self.pp=pp
 		Debug.Print("... DONE")
 	
 	def results_to_zipfile(self,filename=None):
@@ -230,9 +244,9 @@ class Simulation(platrock.Common.Simulations.Simulation):
 		if(filename is None):
 			filename=self.get_dir()+"results.zip"
 		zf = zipfile.ZipFile(filename, "w")
-		pp=Postprocessings.Postprocessing(self)
-		pp.run()
-		outputs=pp.raster.output_to_asc(output_to_string=True)
+		if not self.pp.has_run:
+			self.pp.run()
+		outputs=self.pp.raster.output_to_asc(output_to_string=True)
 		for field,output in outputs.items():
 			zf.writestr(field+".asc",output)
 
@@ -263,7 +277,7 @@ class Simulation(platrock.Common.Simulations.Simulation):
 	
 	def update_checkpoints(self):
 		"""
-		Update all the simulation :class:`ThreeD.Objects.Checkpoint` according to all contacts of all rocks. This is a post-treatement feature, it is supposed to be triggered after the simulation is terminated.
+		Update all the simulation :class:`ThreeD.Objects.Checkpoint` according to all contacts of all rocks. This is a post-processing feature, it is supposed to be triggered after the simulation end.
 		"""
 		for chkP in self.checkpoints:
 			chkP.init_data(self)
@@ -301,10 +315,6 @@ class Simulation(platrock.Common.Simulations.Simulation):
 		self.output.checkpoints=self.checkpoints
 	
 	def before_run_tasks(self):
-		#set the simulation total number of rocks
-		self.nb_rocks=0
-		for rsp in self.rocks_start_params:
-			self.nb_rocks+=rsp['params']["number"]
 		#Offset the position of the start params polygons:
 		if not self._geojson_polygon_offset_applied:
 			for rsp in self.rocks_start_params:
@@ -325,6 +335,7 @@ class Simulation(platrock.Common.Simulations.Simulation):
 			self.terrain.populate_from_Z_raster()
 			self.terrain.precompute_datas()
 			self.automatic_generate_forest["enable"]=True
+			self.__init_arrays__()
 		self.terrain.set_faces_params_from_geojson()
 		if(self.automatic_engines_fill=="builtin"):
 			self.engines=[	Engines.Verlet_update(use_cython=1,dist_factor=5),
@@ -383,7 +394,11 @@ class Simulation(platrock.Common.Simulations.Simulation):
 			self.current_rock=Objects.Parallelepiped(lengths=[l,h,w],pos=pos,density=params["density"])
 			self.current_rock.ori=quaternion.from_euler_angles(self.random_generator.rand(3)*2*np.pi)
 		else:
-			self.current_rock=Objects.Sphere(volume=params["volume"],pos=pos,density=params["density"])
+			self.current_rock=Objects.Sphere(
+				volume=params["rocks_volumes"][self.current_start_params_rock_id-1], #current_start_params_rock_id starts at 1 (why ?)
+				pos=pos,
+				density=params["density"]
+			)
 		#find the face below the rock then set the rock altitude:
 			#1- use the faces bounding boxes in the (x,y) plane to get a rough list
 		mins_check_outside = (pos[0:2]<self.terrain.faces_xyz_bb[:,0:3:2]).any(axis=1)
@@ -410,29 +425,26 @@ class Simulation(platrock.Common.Simulations.Simulation):
 				tree.active=True
 				tree.color=[0,0.8,0]
 	
-	def after_rock_propagation_tasks(self):
-		type(self).__bases__[0].after_rock_propagation_tasks(self)
+	def after_rock_propagation_tasks(self,*args,**kwargs):
+		type(self).__bases__[0].after_rock_propagation_tasks(self,*args,**kwargs)
 		if(self.current_rock.out_of_bounds):
 			self.output.add_contact(self.current_rock,Math.Vector3([0.,0.,0.]),Out.OUT)
 		else:
 			self.output.add_contact(self.current_rock,Math.Vector3([0.,0.,0.]),Out.STOP)
 	
-	def after_successful_run_tasks(self):
-		super(Simulation, self).after_successful_run_tasks()
+	def after_successful_run_tasks(self,*args,**kwargs):
+		super(Simulation, self).after_successful_run_tasks(*args,**kwargs)
 		self.update_checkpoints()
 		if platrock.web_ui:
+			Postprocessings.Postprocessing(self) #this will set self.pp
+			self.pp.run()
 			import platrock.GUI.Plot3D as Plot3D
-			Plot3D.clear()
-			Plot3D.draw_terrain(self,with_polygons=True)
-			Plot3D.draw_sample_trajectories(self,50)
-			Plot3D.plot_checkpoints(self)
-			Plot3D.fig.savefig(self.get_dir()+'terrain_overview.svg',bbox_inches='tight')
-			Plot3D.fig.savefig(self.get_dir()+'terrain_overview.pdf',bbox_inches='tight')
-			Plot3D.plt.close()
+			with open(self.get_dir()+'terrain_overview_plotly.html','w') as f:
+				f.write(Plot3D.get_plotly_raw_html(self,100))
 			self.results_to_zipfile()
 	
-	def after_all_tasks(self):
-		super(Simulation, self).after_all_tasks()
+	def after_all_tasks(self,*args,**kwargs):
+		super(Simulation, self).after_all_tasks(*args,**kwargs)
 
 	def rock_propagation_tasks(self):
 		#Don't call the parent function here as it would slow down a bit. It would be useless as there is nothing in the parent class (nothing common between TwoD and ThreeD)
